{-# LANGUAGE NamedFieldPuns #-}

module Walker.SCAD where

import           Data.Makefile
import           Data.Makefile.Render
import qualified Data.Text            as Text
import           SCAD
import           Shared
import           System.Process
import           Walker
import Data.List.Unique

-- both are not necessary in the PKG
-- A. generate skeletons
-- B. SCAD files

poseToSCAD :: Pose -> [SCAD] -> SCAD
poseToSCAD pose child = Translate (pos pose) [Rotate (ori pose) child]


modelToSCAD :: ExtendedSCADModel -> [SCAD]
modelToSCAD ExtendedSCADModel {absolute_imports=imports, smodel = SCADModel{module_name, args}} = (Use <$> imports) ++ [Module module_name args]

getSCADFiles :: ExtendedWalker -> [(Filename, [SCAD])]
getSCADFiles = mapModels (\x -> (scad_filename x, modelToSCAD x ))

getSCADSkeleton :: ExtendedWalker -> (ExtendedPart -> SCAD) -> [SCAD]
getSCADSkeleton walker part_to_skeleton =  (Use <$> imports) ++ concat (leg_skeleton <$> legs walker) ++ [part_to_skeleton $ body_part $ base_body $ body walker]
    where
        imports = sortUniq $ concat $ mapModels absolute_imports walker

        leg_skeleton :: ExtendedLeg -> [SCAD]
        leg_skeleton Leg {root_joint} = foldJoints fold_joint [] root_joint

        fold_joint :: ExtendedJoint -> [SCAD] -> [SCAD]
        fold_joint ExtendedJoint{joint} sub = [poseToSCAD (offset joint) content]
            where
                lin = link $ sub_link joint

                content :: [SCAD]
                content = case jtype joint of
                    RevoluteJoint {axis, default_angle} -> [Rotate (toQuat axis default_angle) $ (part_to_skeleton <$> parts lin ) ++ sub]
                    FixedJoint -> (part_to_skeleton <$> parts lin) ++ sub


partToVisual :: ExtendedPart -> SCAD
partToVisual Part{model = ExtendedSCADModel {smodel = SCADModel {module_name, args}}} = Module module_name args

partToCollision :: ExtendedPart -> SCAD
partToCollision Part{collision_models} =
  Debug $ (\ExtendedSCADModel {smodel = SCADModel{module_name, args}} -> Module module_name args) <$> collision_models

getSCADVisualSkeleton :: ExtendedWalker ->  [SCAD]
getSCADVisualSkeleton walkie = getSCADSkeleton walkie partToVisual

getSCADCollisionSkeleton :: ExtendedWalker -> [SCAD]
getSCADCollisionSkeleton walkie = getSCADSkeleton walkie partToCollision


getSTLRenderList :: ExtendedWalker -> [(Filename, Filename)]
getSTLRenderList = mapModels model_to_render
    where
        model_to_render :: ExtendedSCADModel -> (Filename, Filename)
        model_to_render ExtendedSCADModel {scad_filename, stl_filename} = (scad_filename, stl_filename)


getMakefileRenderer :: ExtendedWalker -> Makefile
getMakefileRenderer walker = Makefile
  { entries = Rule (Target $ Text.pack "all")
                     (list_to_deb <$> getSTLRenderList walker)
                     []
    : ( list_to_rule <$> getSTLRenderList walker)
  }
 where
  list_to_rule :: (Filename, Filename) -> Entry
  list_to_rule (scad_file, stl_file) = Rule
    (Target $ Text.pack stl_file)
    [Dependency $ Text.pack scad_file]
    (stlRenderCommands scad_file stl_file)

  list_to_deb (_, stl_file) = Dependency $ Text.pack stl_file

renderWalkerStl :: ExtendedWalker -> IO ()
renderWalkerStl walker = do
    writeMakefile "Makefile" $ getMakefileRenderer walker
    callCommand "make"
